/*
 *  Part of rvminis source code,
 *  Copyright (C) 2002 Gabor Varga (bootes@freemail.hu)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "helpfunc.h"
#include "fin.h"








RVFin::RVFin(void)
{
  cnt=0;
  data=NULL;
}



RVFin::~RVFin(void)
{
  delete data;
}



bool RVFin::load(const char *fname)
{
  FILE *f;
  rvulong i;

  if(!filehastype(fname, "fin"))
    {
	  printf("Error: File '%s' is not instance file (.fin).\n",fname);
	  return false;
    }

  f=fopen(fname,"rb");
  if(f==NULL)
    {
      char msg[200];
      sprintf(msg,"Cant read file '%s'",fname);
      perror(msg);
      return false;
    };

  if(!safe_fread(&cnt, sizeof(rvulong), 1, f, fname))
    { fclose(f); return false; }

  delete data;
  data=new S_FIN_Data[cnt];

  for(i=0;i<cnt;i++)
    {
      if(!safe_fread(&data[i], S_FIN_DATA_SIZE, 1, f, fname))
        { fclose(f); return false; }
    }

  fclose(f);
  return true;
}





bool RVFin::save(const char *fname)
{
  FILE *f;
  rvulong i;

  if(!data)
    {
      perror("Internal error.\n");
      return false;
    }

  f=fopen(fname,"wb");
  if(f==NULL)
    {
      char msg[200];
      sprintf(msg,"Cant write file '%s'",fname);
      perror(msg);
      return false;
    };

  fwrite(&cnt, sizeof(rvulong), 1, f);
  for(i=0;i<cnt;i++)
    fwrite(&data[i], S_FIN_DATA_SIZE, 1, f);

  fclose(f);
  return true;
}



bool RVFin::translate(const S_RV_Vector &vec)
{
  rvulong i;

  if(!data)
    {
      perror("Internal error.\n");
      return false;
    }

  for(i=0;i<cnt;i++)
    data[i].position += vec;

  return true;
}

