/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include "instance.h"
#include "helpfunc.h"

int main(int argc, char **argv)
{
  RVInstance rvi;
  char ifile[200], ofile[200];
  
  if((argc<2)||(argc>3))
    {
      printf("Usage: prm2i   input.prm   output.i-p\n\n");
      printf("  Converts the instance file 'input.prm'\n");
      printf("    to the intermediate file 'output.i-p'\n\n");
      printf("Output file name can be omitted, in which case it will be\n");
      printf("derived from the input.\n");
      return 1;
    };

  strcpy(ifile,argv[1]);
  if(!filehastype(ifile,"prm"))
    {
      printf("Error: Input file must be a .prm file\n");
      return 1;
    };

  if(argc < 3)
    strcpy_suffix(ofile,ifile,"i-p");
  else
    {
      strcpy(ofile,argv[1]);
      if(!filehastype(ifile,"i-p"))
	{
	  printf("Error: Output file must be a .i-p file\n");
	  return 1;
	};
    };


  if(!rvi.load(ifile,RV_PRM))
    return 1;

  if(!rvi.save(ofile,IM_PRM))
    return 1;

  printf("Intermediate file '%s' successfully created.\n",ofile);

  return 0;
}
