/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *  Copyright (C) 2001 Gabor Varga (vag) (bootes@freemail.hu)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "instance.h"
#include "helpfunc.h"
#include "rvtypes.h"






/*
 * Most of the stuff herein is a cheap ripoff of some pre-rvglue4-source,
 * and thus too large and confusing for such a small program like
 * this one.
 */


#define TEX_UNKNOWN  -10
#define TEX_SAFETY_DIST (0.5 / 256.0)


class TextureDefinition
{
public:
  TextureDefinition();
  TextureDefinition(rvshort _tex, rvfloat _x0,rvfloat _x1,rvfloat _y0,rvfloat _y1, bool _rot);

  bool uv_in_def(rvshort sheet, S_RV_UV *uv);
  void rotateRight();
  bool isRot() {return rot;}

  //protected:
  rvfloat x0,x1,y0,y1;
  rvshort tex;
  bool rot;	// vag 2001-05-08
};

class ParsedTextureDefinition : public TextureDefinition
{
public:

  void printtex(void);
  int parseTexSpec(const char *str, TextureDefinition *def);


private:
  int parseTexLetter(char *buf);
  int parseSingleNum(char *buf, int *num);
  int parseNumPair(char *obuf, char sepch, int *i1, int *i2, const char *pairname);
};

static TextureDefinition default_src_texdef(TEX_UNKNOWN, 0.0, 1.0, 0.0, 1.0, false);








TextureDefinition::TextureDefinition(rvshort _tex,
				     rvfloat _x0, rvfloat _x1,
				     rvfloat _y0, rvfloat _y1,
                             bool _rot)
{
  x0=_x0;
  y0=_y0;
  x1=_x1;
  y1=_y1;
  tex=_tex;
  rot=_rot;
}

TextureDefinition::TextureDefinition()
{
  x0=y0=0.0;
  x1=y1=1.0;
  tex=TEX_UNKNOWN;
  rot=false;
}

bool TextureDefinition::uv_in_def(rvshort sheet, S_RV_UV *uv)
{
  if((tex != TEX_UNKNOWN) && (sheet != tex))
    return false;

  return (uv->u >= x0-TEX_SAFETY_DIST)
    &&   (uv->u <= x1+TEX_SAFETY_DIST)
    &&   (uv->v >= y0-TEX_SAFETY_DIST)
    &&   (uv->v <= y1+TEX_SAFETY_DIST);
};

void TextureDefinition::rotateRight()   // vag 2001-05-08
{
  rvfloat tx0=x0;
  rvfloat tx1=x1;
  x0=1-y1;
  x1=1-y0;
  y0=tx0;
  y1=tx1;
  rot=true;
}

void ParsedTextureDefinition::printtex(void)
{
  printf("%c.bmp ",
	 tex == TEX_UNKNOWN ? '*' : tex + 'a');

  if(x0 < -1.0)
    printf("(*,*) -> ");
  else
    printf("(%g,%g) -> ",x0*256.0,y0*256.0);

  if(x1 < -1.0)
    printf("(*,*)\n");
  else
    printf("(%g,%g)\n",x1*256.0,y1*256.0);

  if(rot)
    printf("rot\n");
}













int ParsedTextureDefinition::parseTexLetter(char *buf)
{
  if((tolower(buf[0])>='a') && (tolower(buf[0])<='j'))
    tex=tolower(buf[0])-'a';
  else if(buf[0]=='*')
    tex=TEX_UNKNOWN;
  else
    {
      printf("tex letter!\n");
      return 0;
    };

  if(buf[1] != 0)
    printf("ignore tex-lett-trail!\n");

  return 1;
}


int ParsedTextureDefinition::parseSingleNum(char *buf, int *num)
{
  char *c;

  c=buf;
  while((*c >= '0') && (*c <= '9'))
    c++;

  if(*c != 0)
    {
      printf("Number '%s' has non-digit\n",buf);
      return 0;
    };

  if(buf[0]==0)
    {
      printf("Number is empty\n");
      return 0;
    };

  sscanf(buf,"%d",num);

  return 1;
}

int ParsedTextureDefinition::parseNumPair(char *obuf, char sepch, int *i1, int *i2, const char *pairname)
{
  char buf[100];
  char *sep;

  strcpy(buf,obuf);

  if(buf[0]=='*')
    {
      *i1 = *i2 = -1;
      if(buf[1] != 0)
	{
	  printf("ignore trail in '%s'\n",pairname);
	};
    }
  else
    {
      sep=strchr(buf,sepch);
      if(sep == NULL)
	{
	  printf("group has no sep '%s'!\n",pairname);
	  return 0;
	};
      *sep=0; sep++;
      if(!parseSingleNum(buf,i1))
	return 0;
      if(!parseSingleNum(sep,i2))
	return 0;
    };

  return 1;
}


rvfloat px2uv(int i)
{
  return ((rvfloat)i) / 256.0;
}


int ParsedTextureDefinition::parseTexSpec(const char *str, TextureDefinition *def)
{
  char buf[100];
  char *col1, *col2;
  int sx0, sy0, swdt, shgt;

  strcpy(buf,str);

  // vag 2001-05-08
  // check if we have "rot" at the end
  bool rot=false;
  int len=strlen(buf);
  if(len>3 && !strcasecmp(buf+len-3, "rot"))
    {
      rot=true;
      buf[len-3]=0;
    }
  // end vag 2001-05-08

  col1=strchr(buf,':');
  if(col1 == NULL)
    {
      printf("no :\n");
      return 0;
    };
  *col1=0; col1++;

  col2=strchr(col1,':');
  if(col2 == NULL)
    {
      printf("no 2nd :\n");
      return 0;
    };
  *col2=0; col2++;

  if(!parseTexLetter(buf))
    return 0;
  
  if(!parseNumPair(col1, ',', &sx0, &sy0, "top-left corner"))
    return 0;

  if(!parseNumPair(col2, 'x', &swdt, &shgt, "texture size"))
    return 0;


  /* now convert */

  if(sx0 == -1)
    {
      x0=def->x0;
      x1=def->x1;
    }
  else
    {
      x0 = px2uv(sx0);
      y0 = px2uv(sy0);
    };

  if(swdt == -1)
    {
      x1= (def->x1 - def->x0) + x0;
      y1= (def->y1 - def->y0) + y0; 
    }
  else
    {
      x1 = x0 + px2uv(swdt);
      y1 = y0 + px2uv(shgt);
    };

  /* test & fix all values */
  if(x0 > 1.0)
    x0=1.0;

  if(y0 > 1.0)
    y0=1.0;

  if(x1 > 1.0)
    x1=1.0;

  if(y1 > 1.0)
    y1=1.0;


  if((x1 - x0)<0.0001)
    {
      printf("Error: '%s' has a too small size\n",str);
      return 0;
    };
  if((y1 - y0)<0.0001)
    {
      printf("Error: '%s' has a too small size\n",str);
      return 0;
    };

  if(rot)  // vag 2001-05-08
    rotateRight();

  return 1;
}



















/************** APPLICATION PART **************************************/

#define MAX_TDS 100

ParsedTextureDefinition *src_td[MAX_TDS], *dst_td[MAX_TDS];
int num_tds=0;



void free_tdlist(void)
{
  int i;
  for(i=0;i<num_tds;i++)
    {
      delete src_td[i];
      delete dst_td[i];
    };
}

bool add_td(const char *defi)
{
  char src_txt[40], dst_txt[40];
  const char *eqsign;

  eqsign=strchr(defi,'=');
  if(eqsign == NULL)
    {
      printf("Error: Spec '%s' is malformatted, since it contains no '='\n",defi);
      return false;
    };

  strncpy(src_txt, defi, eqsign-defi);
  src_txt[eqsign-defi]=0;
  eqsign++;
  strcpy(dst_txt,eqsign);

  src_td[num_tds] = new ParsedTextureDefinition;
  if(!src_td[num_tds]->parseTexSpec(src_txt, &default_src_texdef))
    {
      delete src_td[num_tds];
      return false;
    };

  // vag 2001-05-08
  if(src_td[num_tds]->isRot())
    {
      printf("Error: Spec '%s' specifies 'rot' for the source, not the target\n", defi);
      delete src_td[num_tds];
      return false;
    }

  dst_td[num_tds] = new ParsedTextureDefinition;
  if(!dst_td[num_tds]->parseTexSpec(dst_txt, src_td[num_tds]))
    {
      delete src_td[num_tds];
      delete dst_td[num_tds];
      return false;
    };

  num_tds++;

  return true;
}






void mod_poly_td(S_RV_Poly *poly, TextureDefinition *src, TextureDefinition *dst)
{
  int npoints= (poly->type & 1) ? 4 : 3;
  int i;
  rvfloat tx, ty;
  rvfloat tmp;

  for(i=0;i<npoints;i++)
    if(!src->uv_in_def(poly->texture, &poly->texcoord[i]))
      return;

  /* is inside, now modify it: */
  if(dst->tex != TEX_UNKNOWN)
    poly->texture = dst->tex;
  
  for(i=0;i<npoints;i++)
    {
      tx = (dst->x1 - dst->x0) / (src->x1 - src->x0);
      ty = (dst->y1 - dst->y0) / (src->y1 - src->y0);

      poly->texcoord[i].u =  dst->x0 + (poly->texcoord[i].u - src->x0)*tx;
      poly->texcoord[i].v =  dst->y0 + (poly->texcoord[i].v - src->y0)*ty;

      if(poly->texcoord[i].u < 0.0) poly->texcoord[i].u = 0.0;
      if(poly->texcoord[i].u > 1.0) poly->texcoord[i].u = 1.0;
      if(poly->texcoord[i].v < 0.0) poly->texcoord[i].v = 0.0;
      if(poly->texcoord[i].v > 1.0) poly->texcoord[i].v = 1.0;

      if(dst->isRot())   // vag 2001-05-08
        {
          // dstdef was rotated right; now rotate the translated uv coords left.
          tmp = poly->texcoord[i].u;
          poly->texcoord[i].u = poly->texcoord[i].v;
          poly->texcoord[i].v = 1 - tmp;          
        }

    };
}


void mod_poly(S_RV_Poly *poly, rvlong *ign_nt, rvlong *ign_nst, S_RV_Vertex *ign_vertices)
{
  int i;

  for(i=0;i<num_tds;i++)
    mod_poly_td(poly,src_td[i],dst_td[i]);
}


int main(int argc, char **argv)
{
  FileType ft;
  RVInstance rvi;
  int i;

  if(argc<3)
    {
      printf("Usage: rvtexmap  filename  [spec]  [spec]  ...\n\n");
      printf("filename may be a .prm or .i-p file.\n\n");
      printf("[spec] is a texture-mapping specification.\n");
      printf("       It's format is described in ReadMe.txt\n\n");
      printf("Example:\n");
      printf("  rvtexmap thing.i-p a:*:*=b:*:* c:0,0:128x128=c:128,128:128x128 d:*:*=e:*:*rot\n\n");
      printf("- modifies the intermediate file 'thing.i-p'\n");
      printf("- remaps texture sheet A to B\n");
      printf("- remaps the 128x128-pixel subtexture at the top-left corner of sheet C\n");
      printf("  to the lower-right corner of the same sheet.\n");
      printf("- remaps texture sheet D to E (rotated left)\n");
      return 1;
    };

  if(filehastype(argv[1],"prm"))
    ft=RV_PRM;
  else if(filehastype(argv[1],"i-p"))
    ft=IM_PRM;
  else
    {
      printf("Error: Name of a .prm or .i-p file expected, not '%s'\n",argv[1]);
      return 1;
    };

  for(i=2;i<argc;i++)
    {
      if(!add_td(argv[i]))
	{
	  free_tdlist();
	  return 1;
	};
    };




  if(!rvi.load(argv[1],ft))
    {
      free_tdlist();
      return 1;
    };

  rvi.apply_modifier(mod_poly);

  if(!rvi.save(argv[1],ft))
    {
      free_tdlist();
      return 1;
    };

  printf("Instance file '%s' successfully modified.\n",argv[1]);

  free_tdlist();

  return 0;
}
