/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include "helpfunc.h"
#include "instance.h"

int main(int argc, char **argv)
{
  RVInstance rvi;
  FileType ft;
  int first, last;
  int i;

  if(argc!=4)
    {
      printf("Usage: rvweird  <filename>  <first>  <last>\n\n");
      printf("Maps parts of instances to new texture sheets.\n\n");
      printf("(see readme.txt for details)\n");
      return 1;
    };

  if((ft=filetype(argv[1]))==Auto)
    {
      printf("Error: File must be an instance file\n");
      return 1;
    };

  if(sscanf(argv[2],"%d",&first)!=1)
    {
      printf("Error: <first> must be a number.\n");
      return 1;
    };
  if(first < 1)
    {
      printf("Error: <first> must be at least 1\n");
      return 1;
    };

  if(sscanf(argv[3],"%d",&last)!=1)
    {
      printf("Error: <last> must be a number.\n");
      return 1;
    };
  if(last < 1)
    {
      printf("Error: <last> must be at least 1\n");
      return 1;
    };

  if(first > last)
    {
      printf("Error: %d is larger than %d. Go get some sleep.\n",first,last);
      return 1;
    };

  if(!rvi.load(argv[1],ft))
    return 1;

  if(first > rvi.header.num_polys)
    {
      printf("Error: '%s' has only %d polygons. How should I start with #%d?\n",
	     argv[1],rvi.header.num_polys,first);
      return 1;
    };
  
  if(last > rvi.header.num_polys)
    last = rvi.header.num_polys;

  first--;
  last--;

  for(i=first;i<=last;i++)
    if(rvi.poly[i].texture >= 0)
      rvi.poly[i].texture = (i - first) % 10;

  if(!rvi.save(argv[1],ft))
    return 1;

  printf("File '%s' modified.\n",argv[1]);

  return 0;
}
